import { CurrencyAmount } from '@sushiswap/sdk';
import { validateAndParseAddress } from "./utils";
import { LimitOrder } from "./order";
import { utils } from 'ethers'

export class FillLimitOrder {
    public readonly order: LimitOrder;
    public readonly path: string[];
    public readonly minimumAmoutOutExternal: CurrencyAmount;
    public readonly amountToFill: CurrencyAmount;
    public readonly limitOrderReceiver: string;
    public readonly to: string;
    public readonly tokenIn: string;
    public readonly tokenOut: string;
    public readonly limitOrderReceiverData: string;


    constructor(order: LimitOrder, pair: string[],  minimumAmoutOutExternal: CurrencyAmount, amountToFill: CurrencyAmount, limitOrderReceiver: string, to: string) {
        this.order = order;

        let validatedPair: string[] = [];

        pair.forEach((p,i) => {
            validatedPair[i] = validateAndParseAddress(p);
        })
        
        this.path = validatedPair;
        this.minimumAmoutOutExternal = minimumAmoutOutExternal;
        this.amountToFill = amountToFill;
        this.limitOrderReceiver = validateAndParseAddress(limitOrderReceiver);
        this.to = validateAndParseAddress(to);
        this.tokenIn = order.amountIn.token.address;
        this.tokenOut = order.amountOut.token.address;

        this.limitOrderReceiverData = utils.defaultAbiCoder.encode(
            ["address[]", "uint256", "address"],
            [this.path, this.minimumAmoutOutExternal.raw.toString(), this.to]
        )
    }
}